/* jzero.c
 * Autor: Joachim Wagner 
 * 25. August 1999 bis 01. September 1999
 * komprimiert 512 Byte Nullbloecke
 */

#include <stdio.h>
#include <stdlib.h>
#include "jzero.h"

int main(int argc, char *argv[])
{
  FILE *in = stdin;
  FILE *out = stdout;
  FILE *temp;
  int size, retcode;
  int rep, repcode, roffset;
  int maxsize = JZERO_BLOCK_GROESSE / sizeof(int);
  int data[(JZERO_BLOCK_GROESSE / sizeof(int)) + 2];
  int format = 4;       /* benutze Bits von ESC fuer repcount */
  int hilfe = 0;
  int fehler = 0;
  int schalter = 1;
  int i, maske;
  maske = maxsize-1;

  while ((schalter) && (argc>1) && (*argv[1]=='-')) {
    switch ((argv[1])[1]) {
      case 'h': hilfe = 1;
                break;
      case 'e': { 
                  char *str = argv[1]+2;
                  if (!*str) {
                    if (argc>3) {
                      argv[1] = argv[0];
                      argv++; 
                      argc--;
                      str = argv[1];
                    }else{
                      fehler = 1;
                    }
                  }
                  if (!fehler) format = atoi(str);
                }
                break;
      case '-': schalter = 0;
                break;
      default:  fprintf(stderr,"Schalter %s unbekannt.\n", argv[1]);
                fehler = 1;
                break;
    }
    argv[1] = argv[0];
    argv++; 
    argc--;
  }
  if (argc>3) fehler = 1;
  if ((format!=0) && (format!=4)) fehler = 1;
  if ((hilfe) || (fehler)) {
    fprintf(stderr,"\nBenutzung: "
                   "%s [Schalter] [--] [Eingabedatei [Ausgabedatei]]\n\n",
                   argv[0]);
    fprintf(stderr,"Schalter:  -h     Gibt diesen Hilfetext aus.\n");
    fprintf(stderr,"           -e 0   Verwendet das alte Format.\n");
    fprintf(stderr,"           -e 4   Verwendet das neue Format (Default).\n");
    fprintf(stderr,"\nJoachim Wagner - 01. September 1999\n");
    exit(fehler);
  }
  if (argc>1) {
    if (!(temp=fopen(argv[1],"rb"))) {
      fprintf(stderr,"Oeffnen der Datei %s fehlgeschlagen.", argv[1]);
      exit(1);
    }
    in = temp;
  }
  if (argc>2) {
    if (!(temp=fopen(argv[2],"wb"))) {
      fprintf(stderr,"Oeffnen der Datei %s fehlgeschlagen.", argv[2]);
      exit(1);
    }
    out = temp;
  }
  if (format) {
    data[0] = JZERO_ESCAPE;
    data[1] = (JZERO_ESCAPE & 0x3fffffff) + 1;  /* Befehlskennzeichen */
    data[2] = 1;  /* Laenge des Befehls */
    for (i=11; i>=2; i--) {
      if (format & (1<<i)) {
        data[3] = 1<<i;
        fwrite(data, 1, 4*sizeof(int), out);
      }
    }
  }
  data[0] = data[1] = JZERO_ESCAPE;
  while ((size = fread(&(data[1]), 1, maxsize * sizeof(int), in))) {
    repcode = data[1];
    data[maxsize+1] = repcode;         /* OK, da unten size geprueft */
    data[maxsize+1]++;
    rep=1;
    if (size == (maxsize * sizeof(int)))
      while (data[rep+1] == repcode) rep++;
    if (rep > 3) {                          /*  && (rep != JZERO_ESCAPE) */
      roffset = (rep-3)+1;
      size -= (rep*sizeof(int));
      size += (3*sizeof(int));
      data[roffset]   = JZERO_ESCAPE;
      data[roffset+1] = rep;
      /* data[roffset+2] = repcode; */   /* steht bereits dort */
    }else{
      if (((format==0) && (repcode == JZERO_ESCAPE)) || 
          ((format==4) && ((repcode & (~maske))
                           == (JZERO_ESCAPE & (~maske))))) {
        roffset = 0;
        size   += sizeof(int);
      }else{
        roffset = 1;
      }
    }
    if ((format & 4) &&             /* Daten ggf. in ESC einbauen */
        (data[roffset]==JZERO_ESCAPE)) {
      if ((data[roffset+1] & (~maske))
            == (JZERO_ESCAPE & (~maske))) { 
        /* nur Daten */
        data[roffset] = 1;
      }else{                              
        roffset++;
        size -= sizeof(int);
        /* Wiederholung 32-Bit Muster */
        /* Max-Anzahl -> 0 */
        data[roffset] &= maske;
        if ((!data[roffset]) && (!data[roffset+1])) { 
          /* Ausnahme: maxsize mal muster 0 */
          roffset++;
          size -= sizeof(int);
          data[roffset] = 2;
        }
      }
      data[roffset] |= (JZERO_ESCAPE & (~maske));
    }
    fwrite(&(data[roffset]), 1, size, out);
    data[0] = data[1] = JZERO_ESCAPE;
  }
  if ( (!feof(in)) || ferror(in) ) {
    fprintf(stderr, "Lesefehler\n");
    retcode = 1;
  }else{
    retcode = 0;
  }  
  if (in  != stdin)  fclose(in);
  if (out != stdout) fclose(out);
  return retcode;
}
